/* ossub.c
 * 2005.12.31
 * 2006.01.06
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <math.h>

#include "fitsio.h"
#include "ossub.h"

int main(int argc, char *argv[])
{
    fitsfile *infptr, *outfptr;  /* FITS file pointers */
    int status = 0, ii, i, j, k;
    //off, level;
    int inaxis;
    int xpos = 0, ypos = 0;
    int x1, x2, x3, x4;
    long inaxes[] = {1,1};
    long npixels = 1, firstpix = 1, check = 1, nflg, ntodo;
    double inpix[FITSBUFF];
    double value, temp;

    double *pixflx;

    printf("# This is %s ver.%s.\n", PROGRAM, VERSION);
    printf("# argc=%d\n", argc);

    if (argc != 7) { 
      printf("Usage: ossub in-image out-image x1 x2 x3 x4\n");
      printf("    in-image:  input fits file name\n");
      printf("   out-image: output fits file name\n");
      printf("       x1,x2: data range (1st pix=1)\n");
      printf("       x3,x4: overscan range to derive bias (median)\n");
      printf("\n");
      return(0);
    }

    /* shift 1pix (in IRAF 1st pix=1) */
    x1 = atoi(argv[3])-1;
    x2 = atoi(argv[4])-1;
    x3 = atoi(argv[5])-1;
    x4 = atoi(argv[6])-1;

    /* check overscan size */
    if (x4-x3 > 255) {
      printf("error: overscansize %d too large.\n", x4-x3);
      exit(EXIT_FAILURE);
    }

    /* open input file */
    fits_open_file(&infptr, argv[1], READONLY, &status);

    fits_get_img_dim(infptr, &inaxis, &status);
    fits_get_img_size(infptr, 2, inaxes, &status);

    /* check axis size */
    printf("# xaxis: %4d   yaxis: %4d\n", inaxes[0], inaxes[1]);
    if (x2 > inaxes[0] || x4 > inaxes[0]) {
      printf("# x-size error.\n");
      exit(EXIT_FAILURE);
    }

    /* memory allocation for fits */
    pixflx = (double*)malloc(inaxes[0]*inaxes[1]*sizeof(double));
    if (pixflx == NULL) {
      printf("# memory allocation error.\n");
      exit(EXIT_FAILURE);
    }
    printf("# %s: memory allocation done.\n", argv[1]);

    for (ii = 0; ii < 2; ii++) {
       npixels *= inaxes[ii];
    }

    nflg = npixels;

    /* create the new empty output file */
    if (check && !fits_create_file(&outfptr, argv[2], &status) )
    {
      /* copy the first input image to the output file */
      fits_copy_header(infptr, outfptr, &status);

      /* operate on FITSBUFF pixels at a time; read as DOUBLE */
      while (nflg && !status)
	{
        if (nflg > FITSBUFF)
          ntodo = FITSBUFF;
        else
          ntodo = nflg;

        fits_read_img(infptr, TDOUBLE, firstpix, ntodo,
                       NULL, inpix, NULL, &status);

	for (ii = 0; ii < ntodo; ii++) {
	  pixflx[xpos+ypos*inaxes[0]] = inpix[ii];
	  xpos++;
	  if (xpos == inaxes[0]) {
	    xpos = 0;
	    ypos++;
	    if (ypos == inaxes[1]) {
	      ypos = 0;
	    }
	  }
	}
	
        firstpix += ntodo;
        nflg  -= ntodo;
      }

      printf("# finished reading images\n");
      /* if error occured, print out error message */
      if (status) fits_report_error(stderr, status);

      /* calculate bias and subtract */
      for (j = 0; j < inaxes[1]; j++) {
	value = 0.;
	for (i = x3; i <= x4; i++) {
	  data1[i-x3] = pixflx[i+j*inaxes[0]];
	}
	for (i = 0; i <= x4-x3; i++) {
	  for (k = x4-x3; k > 0; k--) {
	    if (data1[k] < data1[k-1]) {
	      temp = data1[k-1];
	      data1[k-1] = data1[k];
	      data1[k] = temp;
	    }
	  }
	}
	if ((x4-x3)%2 == 0) {
	  value = (data1[(x4-x3)/2-1] + data1[(x4-x3)/2])/2.;
	} else {
	  value = data1[(x4-x3-1)/2];
	}
	printf("%4d %8.3f\n", j, value);

	for (i = x1; i <= x2; i++) {
	  pixflx[i+j*inaxes[0]] -= value;
	}
      }
      printf("# finished subtraction.\n");
      /* if error occured, print out error message */
      if (status) fits_report_error(stderr, status);

      /* operate on FITSBUFF pixels at a time; write as DOUBLE */
      nflg = npixels;
      firstpix = 1;
      xpos = 0;
      ypos = 0;

      while (nflg && !status) {

	if (nflg > FITSBUFF)
          ntodo = FITSBUFF;
        else
          ntodo = nflg;

	for (ii = 0; ii < ntodo; ii++) {
	  inpix[ii] = pixflx[xpos+ypos*inaxes[0]];
	  xpos++;
	  if (xpos == inaxes[0]) {
	    xpos = 0;
	    ypos++;
	    if (ypos == inaxes[1]) {
	      ypos = 0;
	    }
	  }
	}

        fits_write_img(outfptr, TDOUBLE, firstpix, ntodo,
                       inpix, &status);

        firstpix += ntodo;
        nflg  -= ntodo;
      }

      fits_close_file(outfptr, &status);
    } 

    fits_close_file(infptr, &status);
    free(pixflx);

    printf("# finished writing.\n");

    /* if error occured, print out error message */
    if (status) fits_report_error(stderr, status);
    return(status);
}


